from laceworkreports import common
from laceworkreports.sdk.DataHandlers import ExportHandler, QueryHandler


# cli sets configuration, sdk executes
def export() -> None:
    # connect lacework client
    common.config.connect()

    ExportHandler(
        format=common.config.format,
        results=QueryHandler(
            client=common.config.client,
            type=common.config.TYPE,
            object=common.config.OBJECT,
            filters=common.config.filters,
            returns=common.config.returns,
            lql_query=common.config.lql_query,
            dataset=common.config.dataset,
        ).execute(),
        field_map=common.config.field_map,
        file_path=common.config.file_path,
        template_path=common.config.template_path,
        dtypes=common.config.dtypes,
        db_connection=common.config.db_connection,
        db_table=common.config.db_table,
        db_if_exists=common.config.db_if_exists,
        db_create_if_missing=common.config.db_create_if_missing,
        append=common.config.append,
        flatten_json=common.config.flatten_json,
        sample=common.config.sample,
    ).export()



from enum import Enum

from laceworksdk import LaceworkClient

from laceworkreports.sdk.DataHandlers import DataHandlerTypes

# Environment Variable Definitions for LaceworkSDK
LACEWORK_ACCOUNT_ENVIRONMENT_VARIABLE = "LW_ACCOUNT"
LACEWORK_SUBACCOUNT_ENVIRONMENT_VARIABLE = "LW_SUBACCOUNT"
LACEWORK_API_KEY_ENVIRONMENT_VARIABLE = "LW_API_KEY"
LACEWORK_API_SECRET_ENVIRONMENT_VARIABLE = "LW_API_SECRET"
LACEWORK_API_BASE_DOMAIN_ENVIRONMENT_VARIABLE = "LW_BASE_DOMAIN"
LACEWORK_API_CONFIG_SECTION_ENVIRONMENT_VARIABLE = "LW_PROFILE"

# Environment Variables Definition for LaceworkReports
LACEWORK_REPORTS_DB_CONNECTION = "LW_REPORTS_DB_CONNECTION"
LACEWORK_REPORTS_DB_TABLE = "LW_REPORTS_DB_TABLE"
LACEWORK_REPORTS_SUBACCOUNTS = "LW_REPORTS_SUBACCOUNTS"
LACEWORK_REPORTS_FILE_PATH = "LW_REPORTS_FILE_PATH"
LACEWORK_REPORTS_TEMPLATE_PATH = "LW_REPORTS_TEMPLATE_PATH"


class ActionTypes(Enum):
    Export = "export"
    Format = "format"

    @classmethod
    def has_value(cls, value):
        return value in cls._value2member_map_


class LegacyV2ObjectTypes(Enum):
    AgentAccessTokens = "agent_access_tokens"  # v2 legacy
    AlertChannels = "alert_channels"  # v2 legacy
    AlertRules = "alert_rules"  # v2 legacy
    AuditLogs = "audit_logs"  # v2 legacy
    CloudAccounts = "cloud_accounts"  # v2 legacy
    CloudActivities = "cloud_activities"  # v2 legacy
    ContainerRegistries = "container_registries"  # v2 legacy
    ReportRules = "report_rules"
    ResourceGroups = "resource_groups"
    TeamMembers = "team_members"
    VulnerabilityExceptions = "vulnerability_exceptions"

    @classmethod
    def has_value(cls, value):
        return value in cls._value2member_map_


class ObjectTypes(Enum):
    Entities = "entities"
    Activities = "activities"
    Queries = "queries"
    Vulnerabilities = "vulnerabilities"
    Configs = "configs"
    Alerts = "alerts"
    # add support for legacy v2 which now support paging
    AgentAccessTokens = "agent_access_tokens"  # v2 legacy
    AlertChannels = "alert_channels"  # v2 legacy
    AlertRules = "alert_rules"  # v2 legacy
    AuditLogs = "audit_logs"  # v2 legacy
    CloudAccounts = "cloud_accounts"  # v2 legacy
    CloudActivities = "cloud_activities"  # v2 legacy
    ContainerRegistries = "container_registries"  # v2 legacy
    ReportRules = "report_rules"
    ResourceGroups = "resource_groups"
    TeamMembers = "team_members"
    VulnerabilityExceptions = "vulnerability_exceptions"

    @classmethod
    def has_value(cls, value):
        return value in cls._value2member_map_


class EntitiesTypes(Enum):
    Applications = "applications"
    CommandLines = "command_lines"
    Files = "files"
    NewFileHashes = "new_file_hashes"
    InternalIPs = "internal_ip_addresses"
    K8sPods = "k8s_pods"
    NetworkInterfaces = "network_interfaces"
    Packages = "packages"
    Processes = "processes"
    Users = "users"
    Machines = "machines"
    MachineDetails = "machine_details"
    Containers = "containers"

    @classmethod
    def has_value(cls, value):
        return value in cls._value2member_map_


class ActivitiesTypes(Enum):
    ChangedFiles = "changed_files"
    DNSSummaries = "dns"
    UserLogins = "user_logins"
    Connections = "connections"

    @classmethod
    def has_value(cls, value):
        return value in cls._value2member_map_


class VulnerabilitiesTypes(Enum):
    Hosts = "hosts"
    Containers = "containers"

    @classmethod
    def has_value(cls, value):
        return value in cls._value2member_map_


class ConfigsTypes(Enum):
    ComplianceEvalutations = "compliance_evaluations"

    @classmethod
    def has_value(cls, value):
        return value in cls._value2member_map_


class ComplianceEvaluationsTypes(Enum):
    AwsCompliance = "AwsCompliance"
    AzureCompliance = "AzureCompliance"
    GcpCompliance = "GcpCompliance"

    @classmethod
    def has_value(cls, value):
        return value in cls._value2member_map_


class QueriesTypes(Enum):
    Execute = "execute"

    @classmethod
    def has_value(cls, value):
        return value in cls._value2member_map_


class DBInsertTypes(Enum):
    Append = "append"
    Replace = "replace"
    Fail = "fail"

    @classmethod
    def has_value(cls, value):
        return value in cls._value2member_map_


class Config:
    def __init__(self):
        self.name = __name__.split(".")[0]

        # command context
        self.ACTION = None
        self.TYPE = None
        self.OBJECT = None

        # lacework client
        self.client = None

        # query context
        self.start_time = None
        self.end_time = None
        self.filters = None
        self.returns = None
        self.lql_query = None
        self.dataset = ComplianceEvaluationsTypes.AwsCompliance
        self.sample = False

        # export context
        self.format = DataHandlerTypes.CSV
        self.field_map = None
        self.dtypes = None

        # csv context
        self.file_path = "export.csv"

        # jinja context
        self.template_path = None

        # db context
        self.db_engine = None
        self.db_connection = None
        self.db_table = "export"
        self.db_if_exists = DBInsertTypes.Replace
        self.db_create_if_missing = True

        # format
        self.flatten_json = False
        self.append = False

        # lacework client context
        self.account = None
        self.subaccount = None
        self.api_key = None
        self.api_secret = None
        self.instance = None
        self.profile = None
        self.base_domain = None

        # other
        self.other = "Default"

    def connect(self):
        self.client = LaceworkClient(
            account=self.account,
            subaccount=self.subaccount,
            api_key=self.api_key,
            api_secret=self.api_secret,
            instance=self.instance,
            base_domain=self.base_domain,
            profile=self.profile,
        )

        return self.client


config = Config()
ISO_FORMAT = "%Y-%m-%dT%H:%M:%SZ"


def main():
    pass


if __name__ == "__main__":
    main()



"""laceworkreports is a Python cli/package for creating reports from Lacework data."""

import importlib_metadata


def get_version() -> str:
    try:
        return str(importlib_metadata.version(__name__))
    except importlib_metadata.PackageNotFoundError:  # pragma: no cover
        return "unknown"


version: str = get_version()



"""
LQL Handler
"""

from typing import Optional

from datetime import datetime, timedelta

import typer

from laceworkreports import common
from laceworkreports.sdk.DataHandlers import DataHandlerCliTypes

from .GenericExport import export
from .OptionValidator import update_config, validate

app = typer.Typer()


@app.command(no_args_is_help=True, help="Export to csv")
def csv(
    ctx: typer.Context,
    start_time: datetime = typer.Option(
        (datetime.utcnow() - timedelta(hours=25)).strftime(common.ISO_FORMAT),
        formats=[common.ISO_FORMAT],
        help="Start time for query period",
    ),
    end_time: datetime = typer.Option(
        (datetime.utcnow()).strftime(common.ISO_FORMAT),
        formats=[common.ISO_FORMAT],
        help="End time for query period",
    ),
    lql_query: str = typer.Option(
        ..., help="LQL query string for file path (use @ to specify file path)"
    ),
    field_map: Optional[str] = typer.Option(
        None, help="LQL query string for file path (use @ to specify file path)"
    ),
    file_path: str = typer.Option(
        ...,
        help="Path to exported CSV result",
        envvar=common.LACEWORK_REPORTS_FILE_PATH,
    ),
    append: bool = typer.Option(
        common.config.append, help="Boolean value to append or replace results"
    ),
    flatten_json: bool = typer.Option(
        common.config.flatten_json, help="Boolean value to flatten json result or not"
    ),
) -> None:
    """
    Set the command context
    """

    # command context
    common.config.ACTION = str(ctx.command_path.split(" ")[-4]).replace("-", "_")
    common.config.TYPE = str(ctx.command_path.split(" ")[-3]).replace("-", "_")
    common.config.OBJECT = str(ctx.command_path.split(" ")[-2]).replace("-", "_")
    common.config.format = DataHandlerCliTypes.CSV

    options = validate(
        start_time=start_time,
        end_time=end_time,
        field_map=field_map,
        file_path=file_path,
        lql_query=lql_query,
        flatten_json=flatten_json,
        append=append,
    )

    update_config(options=options)

    # after setting context use sdk to execute
    export()


@app.command(name="json", no_args_is_help=True, help="Export to json")
def json_type(
    ctx: typer.Context,
    start_time: datetime = typer.Option(
        (datetime.utcnow() - timedelta(hours=25)).strftime(common.ISO_FORMAT),
        formats=[common.ISO_FORMAT],
        help="Start time for query period",
    ),
    end_time: datetime = typer.Option(
        (datetime.utcnow()).strftime(common.ISO_FORMAT),
        formats=[common.ISO_FORMAT],
        help="End time for query period",
    ),
    lql_query: str = typer.Option(
        ..., help="LQL query string for file path (use @ to specify file path)"
    ),
    field_map: Optional[str] = typer.Option(
        None, help="LQL query string for file path (use @ to specify file path)"
    ),
    file_path: str = typer.Option(
        ...,
        help="Path to exported JSON result",
        envvar=common.LACEWORK_REPORTS_FILE_PATH,
    ),
    append: bool = typer.Option(
        common.config.append, help="Boolean value to append or replace results"
    ),
) -> None:
    """
    Set the command context
    """

    # command context
    common.config.ACTION = str(ctx.command_path.split(" ")[-4]).replace("-", "_")
    common.config.TYPE = str(ctx.command_path.split(" ")[-3]).replace("-", "_")
    common.config.OBJECT = str(ctx.command_path.split(" ")[-2]).replace("-", "_")
    common.config.format = DataHandlerCliTypes.JSON

    options = validate(
        start_time=start_time,
        end_time=end_time,
        field_map=field_map,
        file_path=file_path,
        lql_query=lql_query,
        append=append,
    )

    update_config(options=options)

    # after setting context use sdk to execute
    export()


@app.command(no_args_is_help=True, help="Export to postgres database")
def postgres(
    ctx: typer.Context,
    start_time: datetime = typer.Option(
        (datetime.utcnow() - timedelta(hours=25)).strftime(common.ISO_FORMAT),
        formats=[common.ISO_FORMAT],
        help="Start time for query period",
    ),
    end_time: datetime = typer.Option(
        (datetime.utcnow()).strftime(common.ISO_FORMAT),
        formats=[common.ISO_FORMAT],
        help="End time for query period",
    ),
    lql_query: str = typer.Option(
        ..., help="LQL query string for file path (use @ to specify file path)"
    ),
    field_map: Optional[str] = typer.Option(
        None,
        help="JSON fieldmap to alias results columns. For file path (use @ to specify file path)",
    ),
    flatten_json: bool = typer.Option(
        common.config.flatten_json, help="Boolean value to flatten json result or not"
    ),
    db_connection: str = typer.Option(
        ...,
        help="Postgres connection string (e.g. postgresql://postgres:password@localhost:5432/postgres)",
        envvar=common.LACEWORK_REPORTS_DB_CONNECTION,
    ),
    db_table: str = typer.Option(
        common.config.db_table,
        help="Postgres table to store results",
        envvar=common.LACEWORK_REPORTS_DB_TABLE,
    ),
    db_if_exists: Optional[common.DBInsertTypes] = typer.Option(
        common.config.db_if_exists.value,
        help="Action to take if db table already exists",
    ),
    db_create_if_missing: Optional[bool] = typer.Option(
        common.config.db_create_if_missing,
        help="Bool to create database if missing",
    ),
) -> None:
    """
    Set the command context
    """

    # command context
    common.config.ACTION = str(ctx.command_path.split(" ")[-4]).replace("-", "_")
    common.config.TYPE = str(ctx.command_path.split(" ")[-3]).replace("-", "_")
    common.config.OBJECT = str(ctx.command_path.split(" ")[-2]).replace("-", "_")
    common.config.format = DataHandlerCliTypes.POSTGRES

    options = validate(
        start_time=start_time,
        end_time=end_time,
        field_map=field_map,
        lql_query=lql_query,
        flatten_json=flatten_json,
        db_connection=db_connection,
        db_table=db_table,
        db_if_exists=db_if_exists,
        db_create_if_missing=db_create_if_missing,
    )

    update_config(options=options)

    # after setting context use sdk to execute
    export()


@app.command(
    no_args_is_help=True, help="Use jinja template to transform export results"
)
def jinja2(
    ctx: typer.Context,
    start_time: datetime = typer.Option(
        (datetime.utcnow() - timedelta(hours=25)).strftime(common.ISO_FORMAT),
        formats=[common.ISO_FORMAT],
        help="Start time for query period",
    ),
    end_time: datetime = typer.Option(
        (datetime.utcnow()).strftime(common.ISO_FORMAT),
        formats=[common.ISO_FORMAT],
        help="End time for query period",
    ),
    lql_query: str = typer.Option(
        ..., help="LQL query string for file path (use @ to specify file path)"
    ),
    field_map: Optional[str] = typer.Option(
        None,
        help="JSON fieldmap to alias results columns. For file path (use @ to specify file path)",
    ),
    file_path: str = typer.Option(
        ...,
        help="Path to exported JINJA2 result",
        envvar=common.LACEWORK_REPORTS_FILE_PATH,
    ),
    flatten_json: bool = typer.Option(
        common.config.flatten_json, help="Boolean value to flatten json result or not"
    ),
    template_path: str = typer.Option(
        ...,
        help="Path to jinja2 template. Results will be passed as 'dataset' variable.",
        envvar=common.LACEWORK_REPORTS_TEMPLATE_PATH,
    ),
) -> None:
    """
    Set the command context
    """

    # command context
    common.config.ACTION = str(ctx.command_path.split(" ")[-4]).replace("-", "_")
    common.config.TYPE = str(ctx.command_path.split(" ")[-3]).replace("-", "_")
    common.config.OBJECT = str(ctx.command_path.split(" ")[-2]).replace("-", "_")
    common.config.format = DataHandlerCliTypes.JINJA2

    options = validate(
        start_time=start_time,
        end_time=end_time,
        field_map=field_map,
        file_path=file_path,
        lql_query=lql_query,
        flatten_json=flatten_json,
        template_path=template_path,
    )

    update_config(options=options)

    # after setting context use sdk to execute
    export()


if __name__ == "__main__":
    app()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['laceworkreports',
 'laceworkreports.cli.ExportHandlers',
 'laceworkreports.cli.ExportHandlers.ActivitiesHandler',
 'laceworkreports.cli.ExportHandlers.ConfigsHandler',
 'laceworkreports.cli.ExportHandlers.DataExportHandlers',
 'laceworkreports.cli.ExportHandlers.EntitiesHandler',
 'laceworkreports.cli.ExportHandlers.QueriesHandler',
 'laceworkreports.cli.ExportHandlers.VulnerabilitiesHandler',
 'laceworkreports.cli.ReportHandlers',
 'laceworkreports.cli.ReportHandlers.AgentCoverageHandler',
 'laceworkreports.cli.ReportHandlers.ComplianceCoverageHandler',
 'laceworkreports.cli.ReportHandlers.ContainerVulnerabilityCoverageHandler',
 'laceworkreports.cli.ReportHandlers.VulnerabilityCoverageHandler',
 'laceworkreports.sdk']

package_data = \
{'': ['*']}

install_requires = \
['Jinja2>=3.0.3,<4.0.0',
 'SQLAlchemy-Utils>=0.38.2,<0.39.0',
 'SQLAlchemy>=1.4.32,<2.0.0',
 'anaconda>=0.0.1,<0.0.2',
 'click==8.0.4',
 'importlib-metadata>=4.10.1,<5.0.0',
 'laceworksdk>=1.1.0,<2.0.0',
 'pandas>=1.4.1,<2.0.0',
 'psycopg2-binary>=2.9.3,<3.0.0',
 'rich>=10.14,<13.0',
 'typer[all]>=0.4.0,<0.5.0']

extras_require = \
{':python_version < "3.8"': ['importlib_metadata>=4.5.0,<5.0.0']}

entry_points = \
{'console_scripts': ['laceworkreports = laceworkreports.__main__:app']}

setup_kwargs = {
    'name': 'laceworkreports',
    'version': '1.3.33',
    'description': 'laceworkreports is a Python cli/package for creating reports from Lacework data.',
    'long_description': '# Lacework Reports CLI/SDK\n\n<div align="center">\n\n[![Build status](https://github.com/laceworkps/laceworkreports/workflows/build/badge.svg?branch=main&event=push)](https://github.com/laceworkps/laceworkreports/actions?query=workflow%3Abuild)\n[![Python Version](https://img.shields.io/pypi/pyversions/laceworkreports.svg)](https://pypi.org/project/laceworkreports/)\n[![Dependencies Status](https://img.shields.io/badge/dependencies-up%20to%20date-brightgreen.svg)](https://github.com/laceworkps/laceworkreports/pulls?utf8=%E2%9C%93&q=is%3Apr%20author%3Aapp%2Fdependabot)\n\n[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)\n[![Security: bandit](https://img.shields.io/badge/security-bandit-green.svg)](https://github.com/PyCQA/bandit)\n[![Pre-commit](https://img.shields.io/badge/pre--commit-enabled-brightgreen?logo=pre-commit&logoColor=white)](https://github.com/laceworkps/laceworkreports/blob/main/.pre-commit-config.yaml)\n[![Semantic Versions](https://img.shields.io/badge/%20%20%F0%9F%93%A6%F0%9F%9A%80-semantic--versions-e10079.svg)](https://github.com/laceworkps/laceworkreports/releases)\n[![License](https://img.shields.io/github/license/laceworkps/laceworkreports)](https://github.com/laceworkps/laceworkreports/blob/main/LICENSE)\n![Coverage Report](assets/images/coverage.svg)\n\nlaceworkreports is a Python cli/package for creating reports from Lacework data.\n\n</div>\n\n## 🚀 Features\n\n- Retrieve Lacework API data from activities, entities, queries, configs\n- Save results as csv, json, or to postgres\n- Transform results using jinja template\n- Override returned field names using field_map (supports nested json notation: parent.child.value)\n- Stores complex json objects as JSONB in postgres\n- Flatten json structures before writing\n\n## CLI Usage\n\n```bash\nlaceworkreports export vulnerabilities hosts csv --file-path="export.csv"\n```\n\n![laceworkreports](assets/images/laceworkreports.gif)\n\nSee [CLI README](README-CLI.md) for details.\n\n## SDK Usage\n\n```python\nfrom laceworkreports import common\nfrom laceworkreports.sdk.DataHandlers import (\n    DataHandlerTypes,\n    ExportHandler,\n    QueryHandler,\n)\n\neh = ExportHandler(\n    format=DataHandlerTypes.CSV,\n    results=QueryHandler(\n        client=LaceworkClient(),\n        type=common.ObjectTypes.Activities.value,\n        object=common.ActivitiesTypes.DNSSummaries.value,\n        filters=[{"field": "mid", "expression": "eq", "value": 851}],\n        returns=["fqdn"],\n    ).execute(),\n    file_path="export.csv",\n).export()\n```\n\nSee [example.py](examples/sdk/example.py) for details.\n\n## Installation\n\n```bash\npip install -U laceworkreports\n```\n\nor install with `Poetry`\n\n```bash\npoetry add laceworkreports\n```\n\nThen you can run\n\n```bash\nlaceworkreports --help\n```\n\nor with `Poetry`:\n\n```bash\npoetry run laceworkreports --help\n```\n\nor run with `docker`:\n\n```bash\ndocker run --rm -it --name laceworkreports \\\n    -v ~/.lacework.toml:/home/user/.lacework.toml -v $(pwd)/reports:/app/reports \\\n    laceworkps/laceworkreports:latest --help\n```\n\nor run with `docker` and start a shell:\n\n```bash\ndocker run --rm --entrypoint="/bin/bash" -it --name laceworkreports \\\n    -v ~/.lacework.toml:/home/user/.lacework.toml -v $(pwd)/reports:/app/reports \\\n    laceworkps/laceworkreports:latest --help\n```\n\nor run with `docker` and force uid:gid on the volume mount (may be required for write permissions):\n\n```bash\ndocker run --rm -it \\\n    --name laceworkreports \\\n    -v ~/.lacework.toml:/home/user/.lacework.toml \\\n    -v $(pwd)/reports:/app/reports \\\n    --env=HOME=/home/user \\\n    --user $UID:$GID \\\n    laceworkps/laceworkreports:latest\n```\n\n## 📈 Releases\n\nYou can see the list of available releases on the [GitHub Releases](https://github.com/laceworkps/laceworkreports/releases) page.\n\nWe follow [Semantic Versions](https://semver.org/) specification.\n\nWe use [`Release Drafter`](https://github.com/marketplace/actions/release-drafter). As pull requests are merged, a draft release is kept up-to-date listing the changes, ready to publish when you’re ready. With the categories option, you can categorize pull requests in release notes using labels.\n\n### List of labels and corresponding titles\n\n|               **Label**               |  **Title in Releases**  |\n| :-----------------------------------: | :---------------------: |\n|       `enhancement`, `feature`        |       🚀 Features       |\n| `bug`, `refactoring`, `bugfix`, `fix` | 🔧 Fixes & Refactoring  |\n|       `build`, `ci`, `testing`        | 📦 Build System & CI/CD |\n|              `breaking`               |   💥 Breaking Changes   |\n|            `documentation`            |    📝 Documentation     |\n|            `dependencies`             | ⬆️ Dependencies updates |\n\n## 🛡 License\n\n[![License](https://img.shields.io/github/license/laceworkps/laceworkreports)](https://github.com/laceworkps/laceworkreports/blob/main/LICENSE)\n\nThis project is licensed under the terms of the `BSD-3` license. See [LICENSE](https://github.com/laceworkps/laceworkreports/blob/main/LICENSE) for more details.\n\n## 📃 Citation\n\n```bibtex\n@misc{laceworkreports,\n  author = {Lacework Inc.},\n  title = {laceworkreports is a Python cli/package for creating reports from Lacework data.},\n  year = {2022},\n  publisher = {GitHub},\n  journal = {GitHub repository},\n  howpublished = {\\url{https://github.com/laceworkps/laceworkreports}}\n}\n```\n',
    'author': 'Lacework Inc.',
    'author_email': 'jamie.mcmurray@lacework.net',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/laceworkps/laceworkreports',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



"""
Example script showing how to use the LaceworkClient class.
"""

import base64
import csv
import json
import logging
import os
import re
from concurrent.futures import ThreadPoolExecutor
from datetime import datetime, timedelta
from enum import Enum
from pathlib import Path

import jinja2
import pandas as pd
import sqlalchemy
from dotenv import load_dotenv
from sqlalchemy import MetaData, Table, create_engine, text
from sqlalchemy_utils.functions import create_database, database_exists

from laceworkreports import common

from .DataHelpers import DataHelpers

logging.basicConfig(
    level=logging.INFO,
    format="%(asctime)s [%(levelname)s] %(filename)s:%(lineno)s - %(message)s",
)

load_dotenv()

ISO_FORMAT = "%Y-%m-%dT%H:%M:%SZ"
LQL_PAGINATION_MAX = 5000
MAX_PSQL_COLUMN_NAME_LENGTH = 63


class DataHandlerTypes(Enum):
    POSTGRES = "postgres"
    SQLITE = "sqlite"
    PANDAS = "pandas"
    DICT = "dict"
    CSV = "csv"
    JSON = "json"
    JINJA2 = "jinja2"

    @classmethod
    def has_value(cls, value):
        return value in cls._value2member_map_


class DataHandlerCliTypes(Enum):
    POSTGRES = "postgres"
    SQLITE = "sqlite"
    CSV = "csv"
    JSON = "json"
    JINJA2 = "jinja2"

    @classmethod
    def has_value(cls, value):
        return value in cls._value2member_map_


class APIv2Helper:
    def __init__(self, session, object_type, endpoint_root="/api/v2"):
        self.session = session
        self._session = session
        self._object_type = object_type
        self._endpoint_root = endpoint_root

    def build_url(self, id=None, resource=None, action=None):
        """
        Builds the URL to use based on the endpoint path, resource, type, and ID.
        :param id: A string representing the ID of an object to use in the URL
        :param resource: A string representing the type of resource to append to the URL
        :param action: A string representing the type of action to append to the URL
        """

        result = f"{self._endpoint_root}/{self._object_type}"

        if resource:
            result += f"/{resource}"
        if action:
            result += f"/{action}"
        if id:
            result += f"/{id}"

        return result

    def search(self, json=None, resource=None, **kwargs):
        response = self._session.post(
            self.build_url(resource=resource, action="search"), json=json
        )
        while True:
            response_json = response.json()
            yield response_json

            try:
                next_page = (
                    response_json.get("paging", {}).get("urls", {}).get("nextPage")
                )
            except Exception:
                next_page = None

            if next_page:
                response = self._session.get(next_page, **kwargs)
            else:
                break


class DataHandler:
    def __init__(
        self,
        format,
        file_path="export.csv",
        template_path=None,
        append=False,
        flatten_json=False,
        dtypes=None,
        db_connection=None,
        db_table="export",
        db_if_exists="replace",
        db_create_if_missing=True,
        sample=False,
    ):
        self.format = format

        # check for valid format type
        if not DataHandlerTypes.has_value(self.format.value):
            raise ValueError(
                "Unsupported export format, exepcted {} found: {}".format(
                    list(DataHandlerTypes), self.format
                )
            )
        else:
            logging.info(f"Processing data with export format: {self.format}")

        # check required args
        if self.format in [DataHandlerTypes.CSV, DataHandlerCliTypes.CSV]:
            if file_path is None:
                raise ValueError(f"file_path required for {self.format} type")
        elif self.format in [DataHandlerTypes.JSON, DataHandlerCliTypes.JSON]:
            if file_path is None:
                raise ValueError(f"file_path required for {self.format} type")
        elif self.format in [DataHandlerTypes.JINJA2, DataHandlerCliTypes.JINJA2]:
            if file_path is None:
                raise ValueError(f"file_path required for {self.format} type")
            if template_path is None:
                raise ValueError(f"template_path required for {self.format} type")
        elif self.format in [DataHandlerTypes.POSTGRES, DataHandlerCliTypes.POSTGRES]:
            if db_connection is None:
                raise ValueError(f"db_connection required for {self.format} type")
            if db_table is None:
                raise ValueError(f"db_table required for {self.format} type")
        elif self.format in [DataHandlerTypes.SQLITE, DataHandlerCliTypes.SQLITE]:
            if db_connection is None:
                raise ValueError(f"db_connection required for {self.format} type")
            if db_table is None:
                raise ValueError(f"db_table required for {self.format} type")

        self.dataset = []
        self.reader = None
        self.file_path = file_path
        self.template_path = template_path
        self.db_connection = db_connection
        self.db_table = db_table
        self.db_if_exists = db_if_exists
        self.db_create_if_missing = db_create_if_missing
        self.dropped_columns = {}
        self.flatten_json = flatten_json
        self.append = append
        self.sample = sample

        # dtypes is the override for sql data types - empty when not provided
        if dtypes is None:
            self.dtypes = {}
        else:
            self.dtypes = dtypes

    def __open(self):
        if self.format in [DataHandlerTypes.CSV, DataHandlerCliTypes.CSV]:
            self.header = False
            self.path = Path(self.file_path)

            # check if we append to existing file or remove
            if Path(self.file_path).exists and not self.append:
                logging.info(f"Removing existing file - append={self.append}")
                self.path.unlink(missing_ok=True)

            if self.append:
                self.fp = open(self.file_path, "a")
            else:
                self.fp = open(self.file_path, "w")

            self.writer = csv.writer(self.fp, quoting=csv.QUOTE_ALL)
        elif self.format in [DataHandlerTypes.JSON, DataHandlerCliTypes.JSON]:
            self.header = False
            self.path = Path(self.file_path)

            # check if we append to existing file or remove
            if Path(self.file_path).exists and not self.append:
                logging.info(f"Removing existing file - append={self.append}")
                self.path.unlink(missing_ok=True)

            if self.append:
                self.fp = open(self.file_path, "a")
            else:
                self.fp = open(self.file_path, "w")

        elif self.format in [DataHandlerTypes.POSTGRES, DataHandlerCliTypes.POSTGRES]:
            try:
                self.db_engine = create_engine(self.db_connection, echo=False)
                logging.info(
                    f'Connecting to "{self.db_engine.url.database}" on port {self.db_engine.url.port} as user "{self.db_engine.url.username}"'
                )
                self.db_engine.connect()
            except sqlalchemy.exc.OperationalError as e:
                if re.match("password authentication failed for user", str(e)):
                    logging.critical(str(e))
                    exit(1)
                elif re.search(r' database "[^"]+" does not exist', str(e)):
                    # check for the db if it doesn't exist create it
                    if (
                        not database_exists(self.db_connection)
                        and self.db_create_if_missing
                    ):
                        logging.info(
                            f"Creating database - db_create_if_missing=True: {self.db_engine.url.database}"
                        )
                        create_database(self.db_engine.url)
                    else:
                        logging.critical(str(e))

                elif re.search(
                    r'could not translate host name "[^"]+" to address: nodename nor servname provided, or not known',
                    str(e),
                ):
                    logging.critical(str(e))
                    raise Exception(str(e))
                elif re.search(
                    r'connection to server at "[^"]+", port \d+ failed: Operation timed out',
                    str(e),
                ):
                    logging.critical(str(e))
                    raise Exception(str(e))

            # connect to the database
            self.conn = self.db_engine.connect()
            metadata = MetaData(bind=self.conn)

            # if we are replacing drop the table first
            if self.db_if_exists == common.DBInsertTypes.Replace.value:
                t = Table(self.db_table, metadata)
                t.drop(self.conn, checkfirst=True)
            elif (
                self.db_if_exists == common.DBInsertTypes.Fail
                and self.db_engine.has_table(self.db_table)
            ):
                logging.error("Table already exists and db_if_exists=fail")
                raise Exception("Table already exists and db_if_exists=fail")
            elif (
                self.db_if_exists == common.DBInsertTypes.Append.value
                and not self.db_engine.has_table(self.db_table)
            ):
                logging.warning(
                    "Database table does not exist and db_if_exists=replace: Table will be created"
                )
        elif self.format in [DataHandlerTypes.SQLITE, DataHandlerCliTypes.SQLITE]:

            # connect to the db
            logging.info(f"Connecting: {self.db_connection}")
            self.db_engine = create_engine(self.db_connection, echo=False)

            # if db doesn't exist create it
            if not database_exists(self.db_engine.url):
                create_database(self.db_engine.url)

            # connect to the database
            self.conn = self.db_engine.connect()

            # drop table if it exists
            metadata = MetaData(bind=self.conn)

            # if we are replacing drop the table first
            if self.db_if_exists == common.DBInsertTypes.Replace:
                t = Table(self.db_table, metadata)
                t.drop(self.conn, checkfirst=True)
            elif (
                self.db_if_exists == common.DBInsertTypes.Fail
                and self.db_engine.has_table(self.db_table)
            ):
                logging.error("Table already exists and db_if_exists=fail")
                raise Exception("Table already exists and db_if_exists=fail")
            elif (
                self.db_if_exists == common.DBInsertTypes.Append
                and not self.db_engine.has_table(self.db_table)
            ):
                logging.warning(
                    "Database table does not exist and db_if_exists=replace: Table will be created"
                )

    def __close(self):
        if self.format in [DataHandlerTypes.CSV, DataHandlerCliTypes.CSV]:
            self.fp.close()

        # for jinja2 we have aggregated into a dict, pass that to the template
        elif self.format in [DataHandlerTypes.JINJA2, DataHandlerCliTypes.JINJA2]:
            report_template = Path(self.template_path).resolve()
            fileloader = jinja2.FileSystemLoader(
                searchpath=os.path.dirname(report_template)
            )
            env = jinja2.Environment(
                loader=fileloader, extensions=["jinja2.ext.do"], autoescape=True
            )
            template = env.get_template(os.path.basename(report_template))
            result = template.render(
                datasets=self.dataset,
                rows=len(self.dataset),
                datetime=datetime,
                timedelta=timedelta,
                config=common.config,
                pandas=pd,
                base64=base64,
            )

            Path(self.file_path).write_text(result)

    def insert(self, row):
        # only flatten json if we're not dumping json
        if self.flatten_json and self.format not in [
            DataHandlerTypes.JSON,
            DataHandlerCliTypes.JSON,
        ]:
            row = DataHelpers.dict_flatten(row)

        if self.format in [DataHandlerTypes.CSV, DataHandlerCliTypes.CSV]:
            if not self.header:
                self.writer.writerow(row.keys())
                self.header = True
            self.writer.writerow(row.values())
        elif self.format in [DataHandlerTypes.JSON, DataHandlerCliTypes.JSON]:
            self.fp.write(f"{json.dumps(row)}\n")
        # if we're doing jinja2 formatting aggregate the result in a dict
        elif self.format in [
            DataHandlerTypes.DICT,
            DataHandlerTypes.JINJA2,
            DataHandlerCliTypes.JINJA2,
        ]:
            self.dataset.append(row)
        elif self.format == DataHandlerTypes.PANDAS:
            if not isinstance(self.dataset, pd.DataFrame):
                self.dataset = pd.DataFrame([row])
            else:
                df = pd.DataFrame([row])
                self.dataset = pd.concat([self.dataset, df], ignore_index=True)
        elif self.format in [DataHandlerTypes.POSTGRES, DataHandlerCliTypes.POSTGRES]:
            try:
                # determine special column handling for json data
                if not self.dtypes:
                    dtypes = {}
                    for k in row.keys():
                        if isinstance(row[k], dict) or isinstance(row[k], list):
                            dtypes[k] = sqlalchemy.types.JSON
                else:
                    dtypes = self.dtypes

                df = pd.DataFrame([row])

                # check for column names that are over max (result of json flatten)
                long_col = [
                    x for x in df.columns if len(x) > MAX_PSQL_COLUMN_NAME_LENGTH
                ]

                long_col_count = len(long_col)
                if long_col_count > 0:
                    # track all dropped column names
                    logging.warning(
                        f"Column {long_col} name length greater than {MAX_PSQL_COLUMN_NAME_LENGTH}, dropping column"
                    )
                    # self.dropped_columns.updated(long_col)
                    df.drop(columns=long_col, inplace=True)

                df.to_sql(
                    self.db_table,
                    if_exists=common.DBInsertTypes.Append.value,
                    index=False,
                    con=self.conn,
                    dtype=dtypes,
                )
            except sqlalchemy.exc.ProgrammingError as e:
                logging.error(e)
                # ensure that any additional columns are added as needed
                for column in df.columns:
                    rows = self.conn.execute(
                        text(
                            "SELECT column_name FROM information_schema.columns WHERE table_name=:db_table and column_name=:column_name"
                        ),
                        table_name=self.db_table,
                        column_name=column,
                    ).fetchall()

                    if len(rows) == 0:
                        logging.debug(
                            f"Unable to find column during insert: {column}; Updating table..."
                        )
                        self.conn.execute(
                            text(
                                "ALTER TABLE :db_table ADD COLUMN :column_name :column_type"
                            ),
                            table_name=self.db_table,
                            column_name=column,
                            column_type=DataHelpers.dataframe_sql_columns(
                                df, column_name=column
                            ),
                        )

                # retry insert with missing columns added
                df.to_sql(
                    self.db_table,
                    if_exists=common.DBInsertTypes.Append.value,
                    index=False,
                    method=None,
                    con=self.conn,
                )
        elif self.format in [DataHandlerTypes.SQLITE, DataHandlerCliTypes.SQLITE]:
            # sync each row of the report to the database
            df = pd.DataFrame([row])
            # determine special column handling for json data
            if not self.dtypes:
                dtypes = {}
                for k in row.keys():
                    if isinstance(row[k], dict) or isinstance(row[k], list):
                        dtypes[k] = sqlalchemy.types.JSON
            else:
                dtypes = self.dtypes

            try:
                df.to_sql(
                    name=self.db_table,
                    con=self.conn,
                    index=False,
                    if_exists=common.DBInsertTypes.Append.value,
                    dtype=dtypes,
                )
            # handle cases where json data has inconsistent rows (add missing here)
            except sqlalchemy.exc.OperationalError as e:
                if re.search(r" table \S+ has no column named", str(e)):
                    ddl = "SELECT * FROM {table_name} LIMIT 1"
                    sql_command = ddl.format(table_name=self.db_table)
                    result = self.conn.execute(text(sql_command)).fetchall()[0].keys()
                    columns = [x for x in result]
                    missing_columns = [x for x in row.keys() if str(x) not in columns]
                    for column in missing_columns:
                        logging.debug(
                            f"Unable to find column during insert: {column}; Updating table..."
                        )

                        # determine the column type
                        if isinstance(row[column], list) or isinstance(
                            row[column], dict
                        ):
                            column_type = "JSON"
                        elif isinstance(row[column], int):
                            column_type = "INTEGER"
                        else:
                            column_type = "TEXT"

                        ddl = "ALTER TABLE {table_name} ADD column {column_name} {column_type}"
                        sql_command = text(
                            ddl.format(
                                table_name=self.db_table,
                                column_name=column,
                                column_type=column_type,
                            )
                        )
                        self.conn.execute(sql_command)

                    # retry adding row
                    df.to_sql(
                        name=self.db_table,
                        con=self.conn,
                        index=False,
                        if_exists=common.DBInsertTypes.Append.value,
                        dtype=dtypes,
                    )
            except Exception as e:
                logging.critical(e)
        else:
            logging.error(f"Unkown format type: {self.format}")

    def show_sample(self, row):
        logging.warn("Sampling only no action will be taken")
        print(json.dumps(row, indent=4))

    def get(self):
        return self.dataset

    def __enter__(self):
        if not self.sample:
            self.__open()

        return self

    def __exit__(self, exc_type, exc_value, traceback):
        if not self.sample:
            self.__close()


class QueryHandler:
    def __init__(
        self,
        client=None,
        type=None,
        object=None,
        start_time=None,
        end_time=None,
        filters=None,
        returns=None,
        lql_query=None,
        dataset=None,
    ):
        # attempt to get context from config
        if client is None:
            client = common.config.client

        if type is None:
            type = common.config.TYPE

        if object is None:
            object = common.config.OBJECT

        if start_time is None:
            start_time = common.config.start_time

        if end_time is None:
            end_time = common.config.end_time

        if filters is None:
            filters = common.config.filters

        if returns is None:
            returns = common.config.returns

        if lql_query is None:
            lql_query = common.config.lql_query

        if dataset is None:
            dataset = common.config.dataset

        # context if not passed or in config
        if start_time is None:
            start_time = datetime.utcnow() + timedelta(days=-1)

        if end_time is None:
            end_time = datetime.utcnow()

        if filters is None:
            filters = []

        self.client = client
        self.type = type
        self.object = object
        self.start_time = start_time
        self.end_time = end_time
        self.filters = filters
        self.returns = returns
        self.lql_query = lql_query
        self.dataset = dataset

    def execute(self):
        # build query string
        q = {
            "timeFilter": {
                "startTime": self.start_time.strftime(ISO_FORMAT),
                "endTime": self.end_time.strftime(ISO_FORMAT),
            },
            "filters": self.filters,
            "returns": self.returns,
        }

        # handle compliance_evalutations dataset field
        if self.dataset is not None:
            q["dataset"] = self.dataset.value

        # create reference to search object
        if common.ObjectTypes.has_value(self.object) and common.ActionTypes.has_value(
            self.type
        ):
            # special case handing where there is only type and no object
            self.type = self.object
            obj = getattr(self.client, f"{self.type}")
        else:
            obj = getattr(getattr(self.client, f"{self.type}"), f"{self.object}")

        # handle lql style queries - limited to LQL_PAGINATION_MAX results
        if common.ObjectTypes.has_value(self.type) and common.QueriesTypes.has_value(
            self.object
        ):
            try:
                response = obj(
                    evaluator_id="<<IMPLICIT>>",
                    query_text=self.lql_query,
                    arguments={
                        "StartTimeRange": self.start_time.strftime(
                            "%Y-%m-%dT%H:%M:%SZ"
                        ),
                        "EndTimeRange": self.end_time.strftime("%Y-%m-%dT%H:%M:%SZ"),
                    },
                )

                result = response.get("data", [])
                logging.debug(f"Query result: {json.dumps(result, indent=4)}")
                num_returned = len(result)
                if num_returned == LQL_PAGINATION_MAX:
                    logging.warning(
                        f"Warning! The maximum number of active containers ({LQL_PAGINATION_MAX}) was returned."
                    )
            except Exception as e:
                logging.error(f"Failed to execute lql query: {e}")
                response = {"data": []}

            return [response]
        elif common.ObjectTypes.has_value(self.type):
            # return query result reference

            # support legacy API functions migrated to v2
            if common.LegacyV2ObjectTypes.has_value(self.type):
                h = APIv2Helper(self.client._session, obj._object_type)
                return h.search(json=q)
            else:
                return obj.search(json=q)
        else:
            logging.error(
                f"Query type {self.type}.{self.object} currently not supported"
            )
            raise Exception(
                f"Query type {self.type}.{self.object} currently not supported"
            )


class ExportHandler:
    def __init__(
        self,
        format,
        results,
        field_map=None,
        dtypes=None,
        file_path="export.csv",
        template_path=None,
        append=False,
        db_connection=None,
        db_table="export",
        db_if_exists="replace",
        db_create_if_missing=True,
        flatten_json=False,
        sample=False,
    ):
        self.format = format
        self.results = results
        self.field_map = field_map
        self.dtypes = dtypes
        self.file_path = file_path
        self.template_path = template_path
        self.append = append
        self.db_connection = db_connection
        self.db_table = db_table
        self.db_if_exists = db_if_exists
        self.db_create_if_missing = db_create_if_missing
        self.flatten_json = flatten_json
        self.sample = sample

    def export(self):
        with DataHandler(
            format=self.format,
            file_path=self.file_path,
            template_path=self.template_path,
            append=self.append,
            db_connection=self.db_connection,
            db_table=self.db_table,
            db_if_exists=self.db_if_exists,
            db_create_if_missing=self.db_create_if_missing,
            flatten_json=self.flatten_json,
            sample=self.sample,
            dtypes=self.dtypes,
        ) as h:
            # process results
            for result in self.results:
                if len(result["data"]) == 0:
                    logging.warn("Query returned 0 results")
                    # raise Exception("Query returned 0 results")
                else:
                    rows = 0
                    if result["data"][0].get("report") is not None:
                        rows = len(result["data"][0].get("report"))
                    else:
                        rows = len(result["data"])

                    logging.info(f"Processing {rows} rows...")
                    for data in result["data"]:
                        # create the data row
                        try:
                            row = DataHelpers.map_fields(
                                data=data, field_map=self.field_map
                            )
                        except Exception as e:
                            logging.error(f"Failed to map fields for data: {data}")
                            raise Exception(e)

                        # sample data and exit
                        if self.sample:
                            h.show_sample(row)
                            exit(1)
                        # thread exporting results
                        else:
                            with ThreadPoolExecutor(max_workers=5) as exe:
                                futures = []
                                future = exe.submit(h.insert, row)
                                futures.append(future)

            # return
            return h.get()



"""
Report Handler
"""

from typing import Optional

import logging
from pathlib import Path

import typer

from laceworkreports import common
from laceworkreports.sdk.DataHandlers import DataHandlerTypes, ExportHandler
from laceworkreports.sdk.ReportHelpers import (
    AWSComplianceTypes,
    AzureComplianceTypes,
    ComplianceQueries,
    GCPComplianceTypes,
    ReportHelper,
)

app = typer.Typer(no_args_is_help=True)


@app.command(no_args_is_help=True, help="Generate HTML report")
def html(
    ctx: typer.Context,
    # current compliance reports do not support time range queries
    # start_time: datetime = typer.Option(
    #     (datetime.utcnow() - timedelta(hours=25)).strftime(common.ISO_FORMAT),
    #     formats=[common.ISO_FORMAT],
    #     help="Start time for query period",
    # ),
    # end_time: datetime = typer.Option(
    #     (datetime.utcnow()).strftime(common.ISO_FORMAT),
    #     formats=[common.ISO_FORMAT],
    #     help="End time for query period",
    # ),
    aws_compliance: AWSComplianceTypes = typer.Option(
        AWSComplianceTypes.AWS_CIS_S3.value,
        help="AWS compliance framework",
    ),
    gcp_compliance: GCPComplianceTypes = typer.Option(
        GCPComplianceTypes.GCP_CIS12.value,
        help="GCP compliance framework",
    ),
    azure_compliance: AzureComplianceTypes = typer.Option(
        AzureComplianceTypes.AZURE_CIS_131.value,
        help="Azure compliance framework",
    ),
    organization: Optional[str] = typer.Option(
        None,
        help="GCP organization id; Required when org level integration is not used",
    ),
    subaccounts: bool = typer.Option(
        False,
        help="Enumerate subaccounts",
        envvar=common.LACEWORK_REPORTS_SUBACCOUNTS,
    ),
    file_path: str = typer.Option(
        ...,
        help="Path to exported result",
        envvar=common.LACEWORK_REPORTS_FILE_PATH,
    ),
    template_path: str = typer.Option(
        Path(__file__).resolve().parent.joinpath("compliance_coverage.html.j2"),
        help="Path to jinja2 template. Results will be passed as 'dataset' variable.",
        envvar=common.LACEWORK_REPORTS_TEMPLATE_PATH,
    ),
    ignore_errors: bool = typer.Option(
        True,
        help="Ignore error for missing reports or inaccessible account details.",
    ),
) -> None:
    """
    Set the command context
    """
    missing_cloud_accounts = []

    # connect lacework client
    lw = common.config.connect()

    # report details
    report_title = "Compliance Coverage"
    db_table = "compliance_coverage"

    reportHelper = ReportHelper()
    db_path = Path("database.db")
    # db_path.unlink(missing_ok=True)
    db_connection = f"sqlite:///{db_path.absolute()}?check_same_thread=False"

    reportHelper.sqlite_drop_table(db_table, db_connection)
    reportHelper.sqlite_drop_table("cloud_accounts", db_connection)

    has_subaccounts = False
    if subaccounts:
        lwAccounts = reportHelper.get_subaccounts(client=lw)
        if len(lwAccounts) == 0:
            logging.error("Subaccounts specificed but none found")
            raise Exception("Subaccounts specificed but none found")
        else:
            has_subaccounts = True
    else:
        lwAccounts = [{"accountName": lw._account}]

    for lwAccount in lwAccounts:
        if has_subaccounts:
            logging.info(f"Switching to subaccount context: {lwAccount['accountName']}")
            lw.set_subaccount(lwAccount["accountName"])

        # get cloud accounts and sync to sqlite
        cloud_accounts = reportHelper.get_cloud_accounts(
            client=lw, lwAccount=lwAccount["accountName"], organization=organization
        )
        ExportHandler(
            format=DataHandlerTypes.SQLITE,
            results=[{"data": cloud_accounts}],
            file_path=file_path,
            db_table="cloud_accounts",
            db_connection=db_connection,
        ).export()

        for cloud_account in cloud_accounts:
            logging.info(
                f"Enumerating {lwAccount['accountName']}:{cloud_account['accountId']}"
            )
            report = reportHelper.get_compliance_report(
                client=lw,
                cloud_account=cloud_account["accountId"],
                lwAccount=lwAccount["accountName"],
                aws_compliance=aws_compliance,
                gcp_compliance=gcp_compliance,
                azure_compliance=azure_compliance,
                ignore_errors=ignore_errors,
                organization=organization,
            )

            if len(report) > 0:
                ExportHandler(
                    format=DataHandlerTypes.SQLITE,
                    results=[{"data": report}],
                    file_path=file_path,
                    db_table=db_table,
                    db_connection=db_connection,
                ).export()
            else:
                missing_cloud_accounts.append(cloud_account["accountId"])

    for miss in missing_cloud_accounts:
        logging.warn(f"missing report for : {miss}")

    # ensure we have a compliance_coverage table
    if not reportHelper.sqlite_table_exists(
        db_table="compliance_coverage", db_connection=db_connection
    ):
        compliance_coverage_table = """
                                    CREATE TABLE compliance_coverage (
                                        "reportType" TEXT, 
                                        "reportTitle" TEXT, 
                                        recommendations JSON, 
                                        summary JSON, 
                                        "projectName" TEXT, 
                                        "organizationName" TEXT, 
                                        "reportTime" TEXT, 
                                        "accountId" TEXT, 
                                        "lwAccount" TEXT
                                    , accountAlias TEXT)
                                    """
        reportHelper.sqlite_execute(
            query=compliance_coverage_table, db_connection=db_connection
        )

    # use sqlite query to generate final result
    results = reportHelper.sqlite_queries(
        queries=ComplianceQueries, db_table=db_table, db_connection=db_connection
    )

    if len(results["report"]) > 0:
        report = results["report"]

        # return additional stats under summary
        stats = {}
        for key in [x for x in results.keys() if x != "report"]:
            stats[key] = results[key]

        # write jinja template
        ExportHandler(
            format=DataHandlerTypes.JINJA2,
            results=[
                {
                    "data": [
                        {
                            "name": db_table,
                            "report": report,
                            "summary": {
                                "rows": len(report),
                                "reportTitle": report_title,
                                "stats": stats,
                            },
                        }
                    ]
                }
            ],
            template_path=template_path,
            file_path=file_path,
        ).export()
    else:
        logging.warn("No report results found.")


@app.command(name="csv", no_args_is_help=True, help="Generate CSV Report")
def csv_handler(
    ctx: typer.Context,
    # current compliance reports do not support time range queries
    # start_time: datetime = typer.Option(
    #     (datetime.utcnow() - timedelta(hours=25)).strftime(common.ISO_FORMAT),
    #     formats=[common.ISO_FORMAT],
    #     help="Start time for query period",
    # ),
    # end_time: datetime = typer.Option(
    #     (datetime.utcnow()).strftime(common.ISO_FORMAT),
    #     formats=[common.ISO_FORMAT],
    #     help="End time for query period",
    # ),
    aws_compliance: AWSComplianceTypes = typer.Option(
        AWSComplianceTypes.AWS_CIS_S3.value,
        help="AWS compliance framework",
    ),
    gcp_compliance: GCPComplianceTypes = typer.Option(
        GCPComplianceTypes.GCP_CIS12.value,
        help="GCP compliance framework",
    ),
    azure_compliance: AzureComplianceTypes = typer.Option(
        AzureComplianceTypes.AZURE_CIS_131.value,
        help="Azure compliance framework",
    ),
    organization: Optional[str] = typer.Option(
        None,
        help="GCP organization id; Required when org level integration is not used",
    ),
    subaccounts: bool = typer.Option(
        False,
        help="Enumerate subaccounts",
        envvar=common.LACEWORK_REPORTS_SUBACCOUNTS,
    ),
    summary_only: bool = typer.Option(
        False,
        help="Return only summary details",
        envvar=common.LACEWORK_REPORTS_SUBACCOUNTS,
    ),
    file_path: str = typer.Option(
        ...,
        help="Path to exported result",
        envvar=common.LACEWORK_REPORTS_FILE_PATH,
    ),
    ignore_errors: bool = typer.Option(
        True,
        help="Ignore error for missing reports or inaccessible account details.",
    ),
) -> None:
    """
    Set the command context
    """
    missing_cloud_accounts = []

    # connect lacework client
    lw = common.config.connect()

    # report details
    db_table = "compliance_coverage"

    reportHelper = ReportHelper()
    db_path = Path("database.db")
    # db_path.unlink(missing_ok=True)
    db_connection = f"sqlite:///{db_path.absolute()}?check_same_thread=False"

    reportHelper.sqlite_drop_table(db_table, db_connection)
    reportHelper.sqlite_drop_table("cloud_accounts", db_connection)

    has_subaccounts = False
    if subaccounts:
        lwAccounts = reportHelper.get_subaccounts(client=lw)
        if len(lwAccounts) == 0:
            logging.error("Subaccounts specificed but none found")
            raise Exception("Subaccounts specificed but none found")
        else:
            has_subaccounts = True
    else:
        lwAccounts = [{"accountName": lw._account}]

    for lwAccount in lwAccounts:
        if has_subaccounts:
            logging.info(f"Switching to subaccount context: {lwAccount['accountName']}")
            lw.set_subaccount(lwAccount["accountName"])

        # get cloud accounts and sync to sqlite
        cloud_accounts = reportHelper.get_cloud_accounts(
            client=lw, lwAccount=lwAccount["accountName"], organization=organization
        )
        ExportHandler(
            format=DataHandlerTypes.SQLITE,
            results=[{"data": cloud_accounts}],
            file_path=file_path,
            db_table="cloud_accounts",
            db_connection=db_connection,
        ).export()

        for cloud_account in cloud_accounts:
            logging.info(
                f"Enumerating {lwAccount['accountName']}:{cloud_account['accountId']}"
            )
            report = reportHelper.get_compliance_report(
                client=lw,
                cloud_account=cloud_account["accountId"],
                lwAccount=lwAccount["accountName"],
                aws_compliance=aws_compliance,
                gcp_compliance=gcp_compliance,
                azure_compliance=azure_compliance,
                ignore_errors=ignore_errors,
                organization=organization,
            )

            if len(report) > 0:
                ExportHandler(
                    format=DataHandlerTypes.SQLITE,
                    results=[{"data": report}],
                    file_path=file_path,
                    db_table=db_table,
                    db_connection=db_connection,
                ).export()
            else:
                missing_cloud_accounts.append(cloud_account["accountId"])

    for miss in missing_cloud_accounts:
        logging.warn(f"missing report for : {miss}")

    # ensure we have a compliance_coverage table
    if not reportHelper.sqlite_table_exists(
        db_table="compliance_coverage", db_connection=db_connection
    ):
        compliance_coverage_table = """
                                    CREATE TABLE compliance_coverage (
                                        "reportType" TEXT, 
                                        "reportTitle" TEXT, 
                                        recommendations JSON, 
                                        summary JSON, 
                                        "projectName" TEXT, 
                                        "organizationName" TEXT, 
                                        "reportTime" TEXT, 
                                        "accountId" TEXT, 
                                        "lwAccount" TEXT
                                    , accountAlias TEXT)
                                    """
        reportHelper.sqlite_execute(
            query=compliance_coverage_table, db_connection=db_connection
        )

    # use sqlite query to generate final result
    results = reportHelper.sqlite_queries(
        queries=ComplianceQueries, db_table=db_table, db_connection=db_connection
    )

    if len(results["report"]) > 0:
        if summary_only:
            report = results["account_coverage"]
        else:
            report = results["report"]

        logging.info("Building CSV from resultant data...")
        ExportHandler(
            format=DataHandlerTypes.CSV,
            results=[{"data": report}],
            file_path=file_path,
        ).export()
    else:
        logging.warn("No report results found.")


if __name__ == "__main__":
    app()



import typer

from laceworkreports import common

from .AgentCoverageHandler import AgentCoverageHandler
from .ComplianceCoverageHandler import ComplianceCoverageHandler
from .ContainerVulnerabilityCoverageHandler import ContainerVulnerabilityCoverageHandler
from .VulnerabilityCoverageHandler import VulnerabilityCoverageHandler

app = typer.Typer(no_args_is_help=True)

commands = [
    {"command_name": "agent-coverage", "command_type": AgentCoverageHandler.app},
    {
        "command_name": "compliance-coverage",
        "command_type": ComplianceCoverageHandler.app,
    },
    {
        "command_name": "vulnerability-coverage",
        "command_type": VulnerabilityCoverageHandler.app,
    },
    {
        "command_name": "container-vulnerability-coverage",
        "command_type": ContainerVulnerabilityCoverageHandler.app,
    },
]

for command in commands:
    app.add_typer(
        command["command_type"],
        name=command["command_name"],
        help=f"Generate {command['command_name']} report",
        no_args_is_help=True,
        epilog=f"{common.config.name} export {command['command_name']} <exporttype> [OPTIONS]",
    )

if __name__ == "__main__":
    app()



import typer

from laceworkreports import common
from laceworkreports.cli.ExportHandlers.DataExportHandlers import GenericAPIv2Handler

app = typer.Typer()

parent_command = common.ActionTypes.Export.value
self_command = common.ObjectTypes[__name__.split(".")[-1]].value

commands = []

for t in common.EntitiesTypes:
    commands.append({"command_name": t.value, "command_type": GenericAPIv2Handler.app})

for command in iter(commands):
    app.add_typer(
        command["command_type"],
        name=command["command_name"],
        help=f"Retrieve lacework activities api {command['command_name']} events",
        no_args_is_help=True,
        epilog=f"{common.config.name} {parent_command} {self_command} {command['command_name']} <exporttype> [OPTIONS]",
    )


if __name__ == "__main__":
    app()
